@@ -1,22 +1,21 @@
 package FileHandle::Fmode;
-use warnings;
-use Fcntl;
+use Fcntl qw(O_RDONLY O_WRONLY O_RDWR F_GETFL);
 use strict;
 
 require Exporter;
 require DynaLoader;
 
-our $VERSION = 0.01;
+$FileHandle::Fmode::VERSION = 0.02;
 
-our @ISA = qw(Exporter DynaLoader);
+@FileHandle::Fmode::ISA = qw(Exporter DynaLoader);
 
-our @EXPORT_OK = qw(is_R is_W is_RO is_WO is_RW);
+@FileHandle::Fmode::EXPORT_OK = qw(is_R is_W is_RO is_WO is_RW);
 
-our %EXPORT_TAGS = (all => [qw
+%FileHandle::Fmode::EXPORT_TAGS = (all => [qw
     (is_R is_W is_RO is_WO is_RW)]);
 
 if($^O =~ /mswin32/i) {
-  bootstrap FileHandle::Fmode $VERSION;
+  bootstrap FileHandle::Fmode $FileHandle::Fmode::VERSION;
 
   *is_R  = \&win32_readable;
   *is_W  = \&win32_writable;
@@ -34,61 +33,101 @@ else {
   }
 
 sub unix_readable_only {
-    my $fmode = fcntl($_[0], F_GETFL, my $slush = 0);
-    if($fmode && $fmode == 0) {return 1}
-    return 0;
+    if(!defined(fileno($_[0]))) {
+      warn "Not an opened filehandle\n";
+      return 0;
     }
+    my $fmode = fcntl($_[0], F_GETFL, my $slush = 0);
+    if(defined($fmode) && $fmode == O_RDONLY) {return 1}
+    return 0;
+}
 
 sub unix_writable_only {
+    if(!defined(fileno($_[0]))) {
+      warn "Not an opened filehandle\n";
+      return 0;
+    }
     my $fmode = fcntl($_[0], F_GETFL, my $slush = 0);
-    if($fmode & 1) {return 1}
+    if($fmode & O_WRONLY) {return 1}
     return 0;
 }
 
 sub unix_writable {
+    if(!defined(fileno($_[0]))) {
+      warn "Not an opened filehandle\n";
+      return 0;
+    }
     my $fmode = fcntl($_[0], F_GETFL, my $slush = 0);
-    if($fmode & 1 || $fmode & 2) {return 1}
+    if($fmode & O_WRONLY || $fmode & O_RDWR) {return 1}
     return 0;
 }
 
 sub unix_readable {
-    my $fmode = fcntl($_[0], F_GETFL, my $slush = 0);
-    if($fmode && ($fmode == 0 || $fmode & 2)) {return 1}
-    return 0;
+    if(!defined(fileno($_[0]))) {
+      warn "Not an opened filehandle\n";
+      return 0;
     }
+    my $fmode = fcntl($_[0], F_GETFL, my $slush = 0);
+    if($fmode && ($fmode == O_RDONLY || $fmode & O_RDWR)) {return 1}
+    return 0;
+}
 
 sub unix_readable_writable {
-    my $fmode = fcntl($_[0], F_GETFL, my $slush = 0);
-    if($fmode & 2) {return 1}
-    return 0;
+    if(!defined(fileno($_[0]))) {
+      warn "Not an opened filehandle\n";
+      return 0;
     }
+    my $fmode = fcntl($_[0], F_GETFL, my $slush = 0);
+    if($fmode & O_RDWR) {return 1}
+    return 0;
+}
 
 sub win32_readable_only {
+    if(!defined(fileno($_[0]))) {
+      warn "Not an opened filehandle\n";
+      return 0;
+    }
     if(win32_fmode($_[0]) & 1) {return 1}
     return 0;
-    }
+}
 
 sub win32_writable_only {
+    if(!defined(fileno($_[0]))) {
+      warn "Not an opened filehandle\n";
+      return 0;
+    }
     if(win32_fmode($_[0]) & 2) {return 1}
     return 0;
 }
 
 sub win32_writable {
+    if(!defined(fileno($_[0]))) {
+      warn "Not an opened filehandle\n";
+      return 0;
+    }
     my $fmode = win32_fmode($_[0]);
     if($fmode & 2 || $fmode & 128) {return 1}
     return 0;
 }
 
 sub win32_readable {
+    if(!defined(fileno($_[0]))) {
+      warn "Not an opened filehandle\n";
+      return 0;
+    }
     my $fmode = win32_fmode($_[0]);
     if($fmode & 1 || $fmode & 128) {return 1}
     return 0;
-    }
+}
 
 sub win32_readable_writable {
+    if(!defined(fileno($_[0]))) {
+      warn "Not an opened filehandle\n";
+      return 0;
+    }
     if(win32_fmode($_[0]) & 128) {return 1}
     return 0;
-    }
+}
 
 1;
 
@@ -114,27 +153,27 @@ FileHandle::Fmode - determine whether a filehandle is opened for reading, writin
 
  $bool = is_R($fh);
  $bool = is_R(\*FH);
-  Returns true if the filehandle is readable.
+  Returns true if the argument is readable.
   Else returns false.
 
  $bool = is_W($fh);
  $bool = is_W(\*FH);
-  Returns true if the filehandle is writable.
+  Returns true if the argument is writable.
   Else returns false
 
  $bool = is_RO($fh);
  $bool = is_RO(\*FH);
-  Returns true if the filehandle is readable but not writable.
+  Returns true if the argument is readable but not writable.
   Else returns false
 
  $bool = is_WO($fh);
  $bool = is_WO(\*FH);
-  Returns true if the filehandle is writable but not readable.
+  Returns true if the argument is writable but not readable.
   Else returns false
 
  $bool = is_RW($fh);
  $bool = is_RW(\*FH);
-  Returns true if the filehandle is both readable and writable.
+  Returns true if the argument is both readable and writable.
   Else returns false
 
 =head1 CREDITS
@@ -142,8 +181,8 @@ FileHandle::Fmode - determine whether a filehandle is opened for reading, writin
 
  Inspired (hmmm ... is that the right word ?) by an idea from BrowserUK
  posted on PerlMonks in response to a question from dragonchild. Win32
- XS code provided by BrowserUK. Not sure who gets the credit for the
- Fcntl macro ... certainly not me.
+ code (including XS code) provided by BrowserUK. Zaxo presented the idea 
+ of using fcntl() in an earlier PerlMonks thread.
 
 =head1 LICENSE
 
